"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Lint = require("tslint");
var chalk_1 = require("chalk");
var logSymbols = require("log-symbols");
var FailureGroup = /** @class */ (function () {
    function FailureGroup(filename) {
        this.filename = filename;
        this.failures = [];
    }
    FailureGroup.prototype.add = function (failure) {
        this.failures.push(failure);
    };
    Object.defineProperty(FailureGroup.prototype, "warningCount", {
        get: function () {
            return this.getCountForSeverity('warning');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FailureGroup.prototype, "errorCount", {
        get: function () {
            return this.getCountForSeverity('error');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FailureGroup.prototype, "fixCount", {
        get: function () {
            return this.failures.reduce(function (count, failure) { return (failure.hasFix() ? count + 1 : count); }, 0);
        },
        enumerable: true,
        configurable: true
    });
    FailureGroup.prototype.getCountForSeverity = function (severity) {
        return this.failures.reduce(function (count, failure) {
            return failure.getRuleSeverity() === severity ? count + 1 : count;
        }, 0);
    };
    return FailureGroup;
}());
var GroupedFailures = /** @class */ (function () {
    function GroupedFailures(failures) {
        var _this = this;
        this.groups = {};
        failures.forEach(function (failure) { return _this.addFailure(failure); });
    }
    GroupedFailures.prototype.addFailure = function (failure) {
        var filename = failure.getFileName();
        var group = this.groups[filename];
        if (!group) {
            this.groups[filename] = group = new FailureGroup(filename);
        }
        group.add(failure);
    };
    Object.defineProperty(GroupedFailures.prototype, "warningCount", {
        get: function () {
            return this.reduce(function (count, group) { return count + group.warningCount; }, 0);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GroupedFailures.prototype, "errorCount", {
        get: function () {
            return this.reduce(function (count, group) { return count + group.errorCount; }, 0);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GroupedFailures.prototype, "fixCount", {
        get: function () {
            return this.reduce(function (count, group) { return count + group.fixCount; }, 0);
        },
        enumerable: true,
        configurable: true
    });
    GroupedFailures.prototype.reduce = function (callbackfn, initialValue) {
        var _this = this;
        return Object.keys(this.groups).reduce(function (previousValue, filename) {
            var group = _this.groups[filename];
            return group ? callbackfn(previousValue, group) : previousValue;
        }, initialValue);
    };
    return GroupedFailures;
}());
var Formatter = /** @class */ (function (_super) {
    __extends(Formatter, _super);
    function Formatter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Formatter.prototype.sortFailures = function (failures) {
        return failures.sort(function (a, b) {
            var _a = a.getStartPosition().getLineAndCharacter(), lineA = _a.line, characterA = _a.character;
            var _b = b.getStartPosition().getLineAndCharacter(), lineB = _b.line, characterB = _b.character;
            var filenameA = a.getFileName();
            var filenameB = b.getFileName();
            if (filenameA === filenameB) {
                return lineA === lineB ? characterA - characterB : lineA - lineB;
            }
            return filenameA < filenameB ? -1 : filenameA > filenameB ? 1 : 0;
        });
    };
    Formatter.prototype.format = function (failures) {
        var failuresByFile = new GroupedFailures(this.sortFailures(failures));
        return getDetails(failuresByFile) + "\n" + getSummary(failuresByFile);
    };
    return Formatter;
}(Lint.Formatters.AbstractFormatter));
exports.Formatter = Formatter;
function formatFailure(failure) {
    var _a = failure.getStartPosition().getLineAndCharacter(), line = _a.line, character = _a.character;
    var position = line + 1 + ":" + (character + 1);
    var message = failure.getFailure();
    var ruleName = failure.getRuleName();
    var severity = failure.getRuleSeverity();
    var severityColor = severity === 'warning' ? chalk_1.default.yellow : chalk_1.default.red;
    var fixHint = failure.hasFix() ? " " + logSymbols.info : '';
    return "  " + severityColor(severity) + ": " + chalk_1.default.blue(position) + " " + message + " " + chalk_1.default.dim(ruleName) + fixHint;
}
function getDetails(failures) {
    return failures.reduce(function (current, group) {
        var headline = chalk_1.default.underline.green(group.filename);
        var formattedFailures = group.failures.map(formatFailure).join('\n');
        return current + "\n" + headline + "\n" + formattedFailures + "\n";
    }, '');
}
function getSummary(failures) {
    var warningCount = failures.warningCount, errorCount = failures.errorCount, fixCount = failures.fixCount;
    var issueCount = warningCount + errorCount;
    var warnings = getCountText('warning', warningCount);
    var errors = getCountText('error', errorCount);
    var status = getStatusSymbol(warningCount, errorCount);
    var summary = status + " Found " + warnings + " and " + errors + ".";
    return issueCount === 0
        ? summary
        : summary + "\n" + getFixableHint(issueCount, fixCount) + "\n";
}
function getFixableHint(issueCount, fixCount) {
    var issues = getCountText('issue', issueCount);
    return logSymbols.info + " " + fixCount + " out of " + issues + " " + (fixCount === 1 ? 'is' : 'are') + " fixable with the tslint option `--fix`.";
}
function getCountText(word, count) {
    return count + " " + (count === 1 ? word : word + "s");
}
function getStatusSymbol(warningCount, errorCount) {
    if (errorCount > 0) {
        return logSymbols.error;
    }
    if (warningCount > 0) {
        return logSymbols.warning;
    }
    return logSymbols.success;
}
